from __future__ import print_function
import glob
import re
import os
from all_classes import *

files = glob.glob('../Box2D/Box2D_*.i')
classes = {}
ignore_files = [
        'Box2D_kwargs.i', 
        'Box2D_dir.i', 
        ]

def find_extended(path):
    f = open(path, "r")
    fn = os.path.split(path)[1]
    for line in f.readlines():
        m=re.search('%extend\s*(.*)\s*{', line)
        if m:
            cls=m.groups()[0].strip()
            if cls in classes:
                if fn not in classes[cls]:
                    classes[cls].append(fn)
            else:
                classes[cls] = [fn]

    f.close()

for file in files:
    if os.path.split(file)[1] not in ignore_files:
        find_extended(file)

print("%19s  %s" % ('Class', 'Extended in'))
remaining=list(all_classes)
for key, value in list(classes.items()):
    print("%20s %s" % (key, ', '.join(value)))
    remaining.remove(key)

ignore_unmodified=[s for s in remaining if s[-3:] == 'Def']
#ignore_unmodified += []

print()
print("Unmodified classes")
for cls in remaining:
    if cls not in ignore_unmodified:
        print(cls)



import os

for platform in ['win32', 'win-amd64']:
    versions=['2.5', '2.6', '2.7', '3.0', '3.1', '3.2']

    def get_path(platform, version):
        if platform=='win32':
            version_path=''
        else:
            version_path='-x64'
        return r'c:\python%s%s\python.exe'%(version.replace('.', ''), version_path)

    interpreters=[get_path(platform, version) for version in versions]
    lib_paths=[r'build\lib.%s-%s' % (platform, version) for version in versions]

    print '@echo off'
    print 'echo ---Start--- > test_results.txt'
    print 'echo ---Start--- > build_results.txt'

    do_stuff='''
echo * %(version)s %(platform)s
echo -------- %(version)s %(platform)s -------- >> build_results.txt
echo -------- %(version)s %(platform)s -------- >> test_results.txt
%(interpreter)s setup.py clean -a >> build_results.txt 2>&1
%(interpreter)s setup.py build --force >> build_results.txt 2>&1

type Box2D\pybox2d_license_header.txt > Box2D_.py
type %(lib_path)s\Box2D\Box2D.py >> Box2D_.py
move /y Box2D_.py %(lib_path)s\Box2D\Box2D.py
%(interpreter)s setup.py develop >> build_results.txt 2>&1
%(interpreter)s setup.py test >> test_results.txt 2>&1
%(interpreter)s setup.py bdist_wininst -t"pybox2d" -dinstaller >> build_results.txt 2>&1
%(interpreter)s setup.py bdist_egg >> build_results.txt 2>&1
    '''

    distutils_cfg='''[build]
compiler=mingw32'''

    for version, interpreter, lib_path in zip(versions, interpreters, lib_paths):
        python_path=os.path.split(interpreter)[0]
        if version < '2.6':
            cfg_file=r'%s\lib\distutils\distutils.cfg' % python_path
            try:
                open(cfg_file, 'w').write(distutils_cfg)
            except:
                pass
        print do_stuff % locals()



from __future__ import print_function

# add "b2ContactPoint", as it's custom

# First, define some classes not seen by swig
ignore = ["b2ContactConstraint", "b2PolygonContact", "b2PolygonAndCircleContact", "b2CircleContact", "b2JointType", "b2BodyType", "b2ContactSolver", "b2PointState", "b2WorldQueryWrapper", "b2WorldRayCastWrapper", "b2SeparationFunction", "b2Block", "b2ContactConstraintPoint", "b2PositionSolverManifold", "b2LimitState"]

# Then check which ones we manually ignore
for line in open('../Box2D/Box2D.i', 'r').readlines():
    if '%ignore' in line:
        c = line.split('ignore ')[1].strip()
        if ';' not in c: 
            continue
        c = c[:c.index(';')].strip()
        ignore.append(c)

all_classes= [
    "b2AABB", 
    "b2Block", 
    "b2BlockAllocator", 
    "b2Body", 
    "b2BodyDef", 
    "b2BodyType", 
    "b2BroadPhase", 
    "b2Chunk", 
    "b2CircleContact", 
    "b2CircleShape", 
    "b2ClipVertex", 
    "b2Color", 
    "b2Contact", 
    "b2ContactPoint", 
    "b2ContactConstraint", 
    "b2ContactConstraintPoint", 
    "b2ContactEdge", 
    "b2ContactFilter", 
    "b2ContactID", 
    "b2ContactImpulse", 
    "b2ContactListener", 
    "b2ContactManager", 
    "b2ContactRegister", 
    "b2ContactSolver", 
    "b2DebugDraw", 
    "b2DestructionListener", 
    "b2DistanceInput", 
    "b2DistanceJoint", 
    "b2DistanceJointDef", 
    "b2DistanceOutput", 
    "b2DistanceProxy", 
    "b2DynamicTree", 
    "b2DynamicTreeNode", 
    "b2Filter", 
    "b2Fixture", 
    "b2FixtureDef", 
    "b2FrictionJoint", 
    "b2FrictionJointDef", 
    "b2GearJoint", 
    "b2GearJointDef", 
    "b2Island", 
    "b2Jacobian", 
    "b2Joint", 
    "b2JointDef", 
    "b2JointEdge", 
    "b2JointType", 
    "b2LimitState", 
    "b2LineJoint", 
    "b2LineJointDef", 
    "b2Manifold", 
    "b2ManifoldPoint", 
    "b2MassData", 
    "b2Mat22", 
    "b2Mat33", 
    "b2MouseJoint", 
    "b2MouseJointDef", 
    "b2Pair", 
    "b2PointState", 
    "b2PolygonAndCircleContact", 
    "b2PolygonContact", 
    "b2PolygonShape", 
    "b2Position", 
    "b2PositionSolverManifold", 
    "b2PrismaticJoint", 
    "b2PrismaticJointDef", 
    "b2PulleyJoint", 
    "b2PulleyJointDef", 
    "b2QueryCallback", 
    "b2RayCastCallback", 
    "b2RayCastInput", 
    "b2RayCastOutput", 
    "b2RevoluteJoint", 
    "b2RevoluteJointDef", 
    "b2Segment", 
    "b2SeparationFunction", 
    "b2Shape", 
    "b2Simplex", 
    "b2SimplexCache", 
    "b2SimplexVertex", 
    "b2StackAllocator", 
    "b2StackEntry", 
    "b2Sweep", 
    "b2TimeStep", 
    "b2TOIInput", 
    "b2Transform", 
    "b2Vec2", 
    "b2Vec3", 
    "b2Velocity", 
    "b2Version", 
    "b2WeldJoint", 
    "b2WeldJointDef", 
    "b2World", 
    "b2WorldManifold", 
    "b2WorldQueryWrapper", 
    "b2WorldRayCastWrapper" ]

# Statistics
print("Total classes: %d" % len(all_classes))
print('Ignored classes:', len(ignore))
for c in ignore:
    if c not in all_classes:
        print('%s not found' % c)
    else:
        all_classes.remove(c)
print('Remaining classes:', len(all_classes))

# And the classes are available in 'all_classes'



classname = "b2DebugDraw"
gets = "GetFlags SetFlags ClearFlags AppendFlags".split(" ")
sets = "".split(" ")
kwargs = True

# remove duplicates
gets = list(set(gets))
sets = list(set(sets))

renames = ["%%rename(__%s) %s::%s;" % (s, classname, s) for s in gets+sets
                    if s not in ('GetAnchorA', 'GetAnchorB', '')]

gets_mod=[]
for s in gets:
    if s[:3]=="Get":
        gets_mod.append(s[3:])
    elif s[:2]=="Is":
        gets_mod.append(s[2:])
    else:
        gets_mod.append(s)

sets_mod=[]
for s in sets:
    if s[:3]=="Set":
        sets_mod.append(s[3:])
    else:
        sets_mod.append(s)

done = []
getter_setter = []
getter = []

for i, s in enumerate(gets_mod):
    if s in sets_mod:
        orig_set=sets[ sets_mod.index(s) ]
        orig_get = gets[i]
        getter_setter.append( (s, orig_get, orig_set) )
        sets[sets_mod.index(s)] = None
    else:
        getter.append( (s, gets[i]) )

setter = [s for s in sets if s is not None]

if kwargs:
    print '''
    /**** %s ****/
    %%extend %s {
    public:
        %%pythoncode %%{
            def __init__(self, **kwargs): 
                """__init__(self, **kwargs) -> %s """
                _Box2D.%s_swiginit(self,_Box2D.new_%s())
                for key, value in kwargs.items():
                    setattr(self, key, value)

            # Read-write properties
''' % tuple([classname[2:]] + [classname]*4)
else:
    print '''
    /**** %s ****/
    %%extend %s {
    public:
        %%pythoncode %%{
            # Read-write properties
''' % (classname[2:], classname)

for name, g, s in getter_setter:
    newname= name[0].lower() + name[1:]
    print "            %s = property(__%s, __%s)" % (newname, g, s)

print "            # Read-only"
for name, g in getter:
    newname= name[0].lower() + name[1:]
    if newname in ('anchorA', 'anchorB'):
        print "            %s = property(lambda self: self._b2Joint__%s(), None)" % (newname, name)
    else:
        print "            %s = property(__%s, None)" % (newname, g)

print "            # Write-only"
for s in setter:
    if not s: continue
    if s[:3]=='Set':
        name = s[3:] 
    else:
        name = s
    newname= name[0].lower() + name[1:]
    print "            %s = property(None, __%s)" % (newname, s)

print """
        %}
    }
"""
print "   ",
print "\n    ".join(renames)



from all_classes import *

print("""
/*
* pybox2d -- http://pybox2d.googlecode.com
*
* Copyright (c) 2010 Ken Lauer / sirkne at gmail dot com
* 
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

%pythoncode %{
    def _init_kwargs(self, **kwargs):
        for key, value in list(kwargs.items()):
            try:
                setattr(self, key, value)
            except Exception as ex:
                raise ex.__class__('Failed on kwargs, class="%s" key="%s": %s' 
                            % (self.__class__.__name__, key, ex))
%}
""")

extend_string="""
%%feature("shadow") %s::%s() {
    def __init__(self, **kwargs):
        _Box2D.%s_swiginit(self,_Box2D.new_%s())
        _init_kwargs(self, **kwargs)
}
"""

director_string="""
%%feature("shadow") %s::%s() {
    def __init__(self, **kwargs):
        if self.__class__ == %s:
            _self = None
        else:
            _self = self
        _Box2D.%s_swiginit(self,_Box2D.new_%s(_self, ))
        _init_kwargs(self, **kwargs)
}
"""
all_classes.remove('b2World')

director_classes = ['b2ContactFilter', 'b2ContactListener', 'b2QueryCallback', 'b2DebugDraw', 'b2DestructionListener' ]
abstract_classes = ['b2Joint', 'b2Shape', 'b2RayCastCallback', 'b2Contact']
for c in abstract_classes:
    # print(director_string % tuple([c]*4))
    all_classes.remove(c)
for c in director_classes:
    print(director_string % tuple([c]*5))
    all_classes.remove(c)
for c in all_classes:
    print(extend_string % tuple([c]*4))



from all_classes import *

print("""
/*
* pybox2d -- http://pybox2d.googlecode.com
*
* Copyright (c) 2010 Ken Lauer / sirkne at gmail dot com
* 
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

%pythoncode %{

    def _dir_filter(self):
        # Using introspection, mimic dir() by adding up all of the __dicts__
        # for the current class and all base classes (type(self).__mro__ returns
        # all of the classes that make it up)
        # Basically filters by:
        # __x__ OK
        # __x bad
        # _classname bad
        def check(s):
            if s[:2]=='__':
                if s[-2:]=='__':
                    return True
                else:
                    return False
            else:
                for typename in typenames:
                    if typename in s:
                        return False
                return True
        
        keys=sum([c.__dict__.keys() for c in type(self).__mro__], [])
        typenames = ["_%s" % c.__name__ for c in type(self).__mro__]
        ret=[s for s in list(set(keys)) if check(s)]
        ret.sort()
        return ret

%}
""")

extend_string="""
%%extend %s {
    %%pythoncode %%{
        __dir__ = _dir_filter
    %%}
}
"""

for c in all_classes:
    print(extend_string % c)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Setup script for pybox2d.

For installation instructions, see INSTALL.

Basic install steps:
 python setup.py build

If that worked, then:
 python setup.py install
"""

import os
import sys
from glob import glob

__author__='Ken Lauer'
__license__='zlib'
__date__="$Date$"
__version__="$Revision$"

import setuptools
from setuptools import (setup, Extension)
setuptools_version = setuptools.__version__
print('Using setuptools (version %s).' % setuptools_version)

if setuptools_version:
    if (setuptools_version in ["0.6c%d"%i for i in range(1,9)] # old versions
        or setuptools_version=="0.7a1"): # 0.7a1 py 3k alpha version based on old version
            print('Patching setuptools.build_ext.get_ext_filename')
            from setuptools.command import build_ext
            def get_ext_filename(self, fullname):
                from setuptools.command.build_ext import (_build_ext, Library, use_stubs)
                filename = _build_ext.get_ext_filename(self,fullname)
                if fullname in self.ext_map:
                    ext = self.ext_map[fullname]
                    if isinstance(ext,Library):
                        fn, ext = os.path.splitext(filename)
                        return self.shlib_compiler.library_filename(fn,libtype)
                    elif use_stubs and ext._links_to_dynamic:
                        d,fn = os.path.split(filename)
                        return os.path.join(d,'dl-'+fn)
                return filename
            build_ext.build_ext.get_ext_filename = get_ext_filename

# release version number
box2d_version  = '2.3'
release_number = 2

# create the version string
version_str = "%s.%s" % (box2d_version, release_number)

# setup some paths and names
library_base='library' # the directory where the egg base will be for setuptools develop command
library_name='Box2D'   # the final name that the library should end up being
library_path=os.path.join(library_base, library_name) # library/Box2D (e.g.)
source_dir='Box2D' # where all of the C++ and SWIG source resides
swig_source='Box2D.i' # the main SWIG source file
use_kwargs=True # whether or not to default creating kwargs for all functions


def write_init():
    # read in the license header
    license_header = open(os.path.join(source_dir, 'pybox2d_license_header.txt')).read()

    # create the source code for the file
    if sys.version_info >= (2, 6):
        import_string = "from .%s import *" % library_name
    else:
        import_string = "from %s import *" % library_name

    init_source = [
        import_string,
        "__author__ = '%s'" % __date__ ,
        "__version__ = '%s'" % version_str,
        "__version_info__ = (%s,%d)" % (box2d_version.replace('.', ','), release_number),
        "__revision__ = '%s'" % __version__,
        "__license__ = '%s'" % __license__ ,
        "__date__ = '%s'" % __date__ , ]

    # and create the __init__ file with the appropriate version string
    f=open(os.path.join(library_path, '__init__.py'), 'w')
    f.write(license_header)
    f.write( '\n'.join(init_source) )
    f.close()

source_paths = [
    os.path.join(source_dir, 'Dynamics'),
    os.path.join(source_dir, 'Dynamics', 'Contacts'),
    os.path.join(source_dir, 'Dynamics', 'Joints'),
    os.path.join(source_dir, 'Common'),
    os.path.join(source_dir, 'Collision'),
    os.path.join(source_dir, 'Collision', 'Shapes'),
    ]

# glob all of the paths and then flatten the list into one
box2d_source_files = [os.path.join(source_dir, swig_source)] + \
    sum( [glob(os.path.join(path, "*.cpp")) for path in source_paths], [])

# arguments to pass to SWIG. for old versions of SWIG, -O (optimize) might not be present.
# Defaults:
# -O optimize, -includeall follow all include statements, -globals changes cvar->b2Globals
# -Isource_dir adds source dir to include path, -outdir library_path sets the output directory
# -small makes the Box2D_wrap.cpp file almost unreadable, but faster to compile. If you want
# to try to understand it for whatever reason, I'd recommend removing that option.
swig_arguments = \
        '-c++ -I%s -small -O -includeall -ignoremissing -w201 -globals b2Globals -outdir %s' \
        % (source_dir, library_path)

if use_kwargs:
    # turn off the warnings about functions that can't use kwargs (-w511)
    # and let the wrapper know we're using kwargs (-D_SWIG_KWARGS)
    swig_arguments += " -keyword -w511 -D_SWIG_KWARGS"

# depending on the platform, add extra compilation arguments. hopefully if the platform
# isn't windows, g++ will be used; -Wno-unused then would suppress some annoying warnings
# about the Box2D source.
if sys.platform in ('win32', 'win64'):
    extra_args=['-I.']
else:
    extra_args=['-I.', '-Wno-unused']

pybox2d_extension = \
    Extension('Box2D._Box2D', box2d_source_files, extra_compile_args=extra_args, language='c++')

LONG_DESCRIPTION = \
""" 2D physics library Box2D %s for usage in Python.

    After installing please be sure to try out the testbed demos.
    They require either pygame or pyglet and are available on the
    homepage.

    pybox2d homepage: http://pybox2d.googlecode.com
    Box2D homepage: http://www.box2d.org
    """ % (box2d_version,)

CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: zlib/libpng License",
    "Operating System :: Microsoft :: Windows",
    "Operating System :: MacOS :: MacOS X",
    "Operating System :: POSIX",
    "Programming Language :: Python",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Topic :: Software Development :: Libraries :: pygame",
    ]

write_init()

setup_dict = dict(
    name             = "Box2D",
    version          = version_str,
    author           = "Ken Lauer",
    author_email     = "sirkne at gmail dot com",
    description      = "Python Box2D",
    license          = "zlib",
    url              ="http://pybox2d.googlecode.com/",
    long_description = LONG_DESCRIPTION,
    classifiers      = CLASSIFIERS,
    packages         = ['Box2D', 'Box2D.b2'],
    package_dir      = {'Box2D': library_path,
                        'Box2D.b2': os.path.join(library_path, 'b2'),
                        'Box2D.tests' : 'tests'},
    test_suite       = 'tests',
    options          = { 'build_ext': { 'swig_opts' : swig_arguments },
                         'egg_info' : { 'egg_base' : library_base },
                        },
    ext_modules      = [ pybox2d_extension ],
#   use_2to3         = (sys.version_info >= (3,)),
    )

# run the actual setup from distutils
setup(**setup_dict)
